@@ -1,28 +1,40 @@
 package App::Mobirc::Plugin::HTMLFilter::DoCoMoCSS;
 use strict;
-use MooseX::Plaggerize::Plugin;
+use warnings;
 use CSS::Tiny;
 use XML::LibXML;
 use HTML::Selector::XPath;
 use App::Mobirc::Util;
-use Encode;
-use Path::Class;
 
-# some code copied from HTML::DoCoMoCSS
-hook 'html_filter' => sub {
-    my ($self, $global_context, $c, $content) = @_;
+sub register {
+    my ($class, $global_context) = @_;
+
+    DEBUG "Register DoCoMoCSS";
+
+    $global_context->register_hook(
+        'html_filter' => \&_html_filter_docomocss,
+    );
+}
+
+# based from HTML::DoCoMoCSS
+sub _html_filter_docomocss {
+    my $c = shift;
+    my $content = shift;
 
     DEBUG "FILTER DOCOMO CSS";
-    return ($c, $content) unless $c->req->mobile_agent->is_docomo;
+    return $content unless $c->{mobile_agent}->is_docomo;
 
     # escape Numeric character reference.
     $content =~ s/&#(x[\dA-Fa-f]{4}|\d+);/HTMLCSSINLINERESCAPE$1::::::::/g;
     # unescape Numeric character reference.
     my $pict_unescape = sub { $content =~ s/HTMLCSSINLINERESCAPE(x[\dA-Z-a-z]{4}|\d+)::::::::/&#$1;/g; return $content; };
 
-    my $css = CSS::Tiny->read_string($self->css_text($global_context));
+    $content =~ s{<style type="text/css">(.+)</style>}{}sm;
+    my $css_text = $1 or return $pict_unescape->();
+
+    my $css = CSS::Tiny->read_string($css_text);
     my $doc = eval { XML::LibXML->new->parse_string($content); };
-    $@ and return ($c, $pict_unescape->());
+    $@ and return $pict_unescape->();
 
     # apply inline css
     while (my($selector, $style) = each %{ $css }) {
@@ -34,17 +46,11 @@ hook 'html_filter' => sub {
             $element->setAttribute('style', $style_attr);
         }
     }
-    $content = decode_utf8($doc->toString);
+    $content = $doc->toString;
 
     $content =~ s{(<a[^>]+)/>}{$1></a>}gi;
 
-    return ($c, $pict_unescape->());
-};
-
-sub css_text {
-    my ($self, $global_context) = @_;
-    my $root = dir($global_context->config->{global}->{assets_dir}, 'static');
-    $root->file('mobirc.css')->slurp . "\n" . $root->file('mobile.css')->slurp;
+    return $pict_unescape->();
 }
 
 1;
